#!/usr/bin/python
from array import *
from subprocess import call
import ConfigParser
import shutil
import re
import rrdtool
import os

# read configuration file 
config = ConfigParser.RawConfigParser()
config.read(os.path.dirname(os.path.realpath(__file__))+'/rrdconfig.ini')
graphs = (config.get('config', 'GRAPHS')).split(',')
nsnames = (config.get('config', 'NSNAMES')).split(',')
domainname = config.get('config', 'DOMAIN')
rndcbin = config.get('config', 'RNDCBIN')
rrdtoolbin = config.get('config', 'RRDTOOLBIN')
rndcport = config.get('config', 'RNDCPORT')
nsrootpath = config.get('config', 'NSROOTPATH')
nslogpath = config.get('config', 'NSLOGPATH')
nsrrdpath = config.get('config', 'NSRRDPATH')
nsxmlpath = config.get('config', 'NSXMLPATH')
statsfile = config.get('config', 'STATSFILE')

for nsname in nsnames:
	nsnamefqdn = nsname + domainname
	statsfile_fn = nsrootpath + '/' + nslogpath + '/' + nsname + '/' + statsfile
	statsfile_bkp_fn = statsfile_fn + '.0'
	shutil.copyfile(statsfile_fn,statsfile_bkp_fn)

	call([rndcbin , '-s' , nsnamefqdn , '-p' , rndcport , 'stats'])

	statslist = open(statsfile_fn).readlines()

	for graph in graphs:
		graph_items = (config.get(graph, 'ITEMS')).split(',')
		start_delim = config.get(graph, 'START_DELIM')
		end_delim = config.get(graph, 'END_DELIM')
		value_index = int(config.get(graph, 'INDEX'))
		rrd_template = config.get(graph, 'RRDTEMPLATE')
		timespans = (config.get(graph, 'TIMESPANS')).split(',')
		graph_width = config.get(graph, 'WIDTH')
		graph_height = config.get(graph, 'HEIGHT')
		vertical_legend = config.get(graph, 'VLEGEND')
		image_folder = config.get(graph, 'IMAGEFOLDER')
		
		parsingflag = False
		result_arr = [0] * len(graph_items)
		for line in statslist:
			if line.startswith(start_delim):
				parsingflag = True
			elif line.startswith(end_delim):
				parsingflag = False
			if parsingflag:
				i = 0
				for graph_item in graph_items:
					regex_pattern = '[\\b0-9]* '+graph_item+'$'
					if re.search(regex_pattern,line,re.IGNORECASE):
						value = line.lower().split(graph_item.lower())[value_index].strip()
						result_arr[i] = value
					i += 1
		output = "N:" + ":".join(map(str,result_arr))
		rrdfilename_fn = nsrootpath + '/' + nsrrdpath + "/" + graph + nsname + ".rrd"
		
		if not os.path.exists(rrdfilename_fn):
			rrdtool_create_param = []
			rrdtemplate_arr = rrd_template.split(':')
			for rrd_temp in rrdtemplate_arr:
				rrdtool_create_param.extend(["DS:" + rrd_temp + ":COUNTER:600:0:10000000"])
			rrdtool.create( rrdfilename_fn,
					'--step', '300',
					rrdtool_create_param,
					'RRA:AVERAGE:0.5:1:800',
					'RRA:AVERAGE:0.5:6:800',
					'RRA:AVERAGE:0.5:24:800',
					'RRA:AVERAGE:0.5:288:800',
					'RRA:MAX:0.5:1:800',
					'RRA:MAX:0.5:6:800',
					'RRA:MAX:0.5:24:800',
					'RRA:MAX:0.5:288:800' )

		rrdtool.update(rrdfilename_fn, '--template', rrd_template, output)

		for timespan in timespans:
			timespan_short = timespan[0:1].lower()
			image = nsrootpath + '/' + image_folder + "/" + graph + "_" + nsname + "_" + timespan + ".png" 
			title = nsname + " (" + timespan + ")"
			rrdtool_graph_param = []
			rrdtool_xml_param = []	
			rrdtemplate_arr = rrd_template.split(':')
			
			string_length = 0
			string_length_tmp = 0
			string_length_prv = 0
			for rrd_temp in rrdtemplate_arr:
				string_length_tmp = len(rrd_temp)
				if (string_length_tmp > string_length):
					string_length = string_length_tmp
				string_length_prv = string_length_tmp

			c = 0
			colors = (config.get('config', 'COLORS')).split(',')
			l_increment = "a"
			for rrdtemplate_item in rrdtemplate_arr:
				align_chr_length = string_length - len(rrdtemplate_item)
				sep_chr = ' ' * align_chr_length
				
				rrdtool_graph_param.extend(["DEF:" + l_increment + "=" + rrdfilename_fn + ":" + rrdtemplate_item + ":AVERAGE"])
				rrdtool_graph_param.extend(["CDEF:cdef" + l_increment + "=" + l_increment + ",60,*"])
				rrdtool_graph_param.extend(["LINE1:cdef" + l_increment + colors[c] + ":" + rrdtemplate_item.upper()])
				rrdtool_graph_param.extend(["GPRINT:cdef" + l_increment + ":LAST:" + sep_chr + " Cur\:%6.1lf"])
				rrdtool_graph_param.extend(["GPRINT:cdef" + l_increment + ":AVERAGE: Ave\:%6.1lf"])
				rrdtool_graph_param.extend(["GPRINT:cdef" + l_increment + ":MAX: Max\:%6.1lf\\n"])

				rrdtool_xml_param.extend(["DEF:" + l_increment + "=" + rrdfilename_fn + ":" + rrdtemplate_item + ":AVERAGE"])
				rrdtool_xml_param.extend(["XPORT:" + l_increment + ":" + rrdtemplate_item])

				l_increment = chr(ord(l_increment) + 1)
				c += 1

			timespan_param = '-1' + timespan_short

			xmlfile = nsrootpath + '/' + nsxmlpath + "/" + graph + "_" + nsname + "_" + timespan + ".xml"
			rrdtoolxport = [rrdtoolbin , 'xport' , "-s" , timespan_param] + rrdtool_xml_param			
			f = open(xmlfile,"wb")
			call(rrdtoolxport ,stdout=f)
			rrdtool.graph(image,
					'--imgformat', 'PNG',
					'--width', graph_width,
					'--height', graph_height,
					'--vertical-label', vertical_legend,
					'--slope-mode',
					'--start', timespan_param,
					'--title', title, 
					rrdtool_graph_param)		
